@@ -9,7 +9,7 @@ class SLRParser:
     def __init__(self, G):
         self.G_prime = Grammar(f"{G.start}' -> {G.start}\n{G.grammar_str}")
         self.max_G_prime_len = len(max(self.G_prime.grammar.keys(), key=len))
-        self.G_indexed = [['', '']]
+        self.G_indexed = []
 
         for head, bodies in self.G_prime.grammar.items():
             for body in bodies:
@@ -17,8 +17,9 @@ class SLRParser:
 
         self.first, self.follow = self.first_follow(self.G_prime)
         self.C = self.items(self.G_prime)
-        self.parse_table_symbols = list(self.G_prime.terminals) + ['$'] + list(
-            self.G_prime.nonterminals - {self.G_prime.start})
+        self.action = list(self.G_prime.terminals) + ['$']
+        self.goto = list(self.G_prime.nonterminals - {self.G_prime.start})
+        self.parse_table_symbols = self.action + self.goto
         self.parse_table = self.construct_table()
 
     def first_follow(self, G):
@@ -99,9 +100,15 @@ class SLRParser:
                     dot_pos = body.index('.')
 
                     if body[dot_pos + 1] == X:
-                        for C_head, C_bodies in self.CLOSURE(
-                            {head: [body[:dot_pos] + [X, '.'] + body[dot_pos + 2:]]}).items():
-                            goto[C_head] = C_bodies
+                        replaced_dot_body = body[:dot_pos] + [X, '.'] + body[dot_pos + 2:]
+
+                        for C_head, C_bodies in self.CLOSURE({head: [replaced_dot_body]}).items():
+                            if C_head not in goto.keys():
+                                goto[C_head] = C_bodies
+                            else:
+                                for C_body in C_bodies:
+                                    if C_body not in goto[C_head]:
+                                        goto[C_head].append(C_body)
 
         return goto
 
@@ -166,6 +173,9 @@ class SLRParser:
         def print_line():
             print(f'+{("-" * width + "+") * (len(list(self.G_prime.symbols) + ["$"]))}')
 
+        def symbols_width(symbols):
+            return (width + 1) * len(symbols) - 1
+
         print('AUGMENTED GRAMMAR:')
 
         i = 0
@@ -197,10 +207,10 @@ class SLRParser:
 
         print('\nPARSING TABLE:')
         print(
-            f'+{"-" * width}+{"-" * ((width + 1) * len(list(self.G_prime.terminals) + ["$"]) - 1)}+{"-" * ((width + 1) * (len(self.G_prime.nonterminals) - 1) - 1)}+')
+            f'+{"-" * width}+{"-" * symbols_width(self.action)}+{"-" * symbols_width(self.goto)}+')
         print(
-            f'|{"":{width}}|{"ACTION":^{(width + 1) * len(list(self.G_prime.terminals) + ["$"]) - 1}}|{"GOTO":^{(width + 1) * (len(self.G_prime.nonterminals) - 1) - 1}}|')
-        print(f'|{"STATE":^{width}}+{("-" * width + "+") * (len(list(self.G_prime.symbols) + ["$"]) - 1)}')
+            f'|{"":{width}}|{"ACTION":^{symbols_width(self.action)}}|{"GOTO":^{symbols_width(self.goto)}}|')
+        print(f'|{"STATE":^{width}}+{("-" * width + "+") * len(self.parse_table_symbols)}')
         print(f'|{"":^{width}}|', end=' ')
 
         for symbol in self.parse_table_symbols:
